#include "head.h"
int sockfd;
struct sockaddr_in servaddr;
const char *R_FILE_NAME  = NULL;
const char *W_FILE_NAME  = NULL;
u8 file_buffer[256];

static const struct option long_options[] =
{
	{ "Input  ServerIP",           required_argument, NULL, 'i' },
	{ "Input  ServerPort",         required_argument, NULL, 'p' },
	{ "change ServerIP",           required_argument, NULL, 'I' },
	{ "change ServerPort",         required_argument, NULL, 'P' },
	{ "change Mac",                required_argument, NULL, 'M' }, 
	{ "change TsPort",             required_argument, NULL, 'T' }, 
    { "Multicast Enable",          required_argument, NULL, 'E' },
    { "Multicast Disenable",       no_argument,       NULL, 'D' },
    { "write firmware to flash",   required_argument, NULL, 'w' },
    { "read  firmware from flash", required_argument, NULL, 'r' },
    { "check  firmware's version", no_argument,       NULL, 'v' },
	{ "check  networktool's version", no_argument,    NULL, 'd' },
	{ 0, 0, 0, 0 }
};
  

void usage()
{
	printf("\n");	
	printf("Usage: networktool:\n");
	printf("-i <server ip> -p <server port>\n");
	printf("-I <new server ip> -P <new server port>\n");
  //  printf("-i  : input Server IP,eg:192.168.8.188\n");
  //  printf("-p  : input server Port(0~65535),eg:5442\n");
  //  printf("-I  : change new Server ip\n");
  //  printf("-P  : change new server port\n");
    printf("-M  : change new MAC ,and showed by hex,eg: xx:xx:xx:xx:xx:xx\n");
    printf("-T  : change new TsPort(0~65535)\n");
    printf("-E  : enable and change Multicast,eg:*.*.*.*\n");
    printf("-D  : disenable Multicast\n");
    printf("-w  : write firmware to flash,eg: ./networktool -w xxx.bin\n");
    printf("-r  : read firmware form flash,eg: ./networktool -r xxx.bin\n");
    printf("-v  : check firmware's version\n");
	printf("-d  : check  networktool's version\n");
    printf("\n\n");
}     


static void DisplayVersion()
{
    u8 flash_id[8] = {0};   
    if(spi_GetFlashID(flash_id) == 0){
        printf("FW Version : 0x%x%x\n",flash_id[5],flash_id[6]);//show version number
    } else 
            printf("get version failed !!\n");
    
    exit(0);
}

static void Read_Flash()
{
    u32 addr;
    FILE *r_file_fd;
    u8 spi24bitAddr[4] = {0};
    u8 val = 0;
    r_file_fd = fopen(R_FILE_NAME,"wb");
    fseek(r_file_fd,0,SEEK_SET);
    if(r_file_fd == NULL){
        printf("errno open file!!\n");
        exit(1);
    }else{
        printf("File Open successed!\n");
    }
    printf("read wait ...\n");
    for(addr=0;addr<0x400000;addr+=4){
        u32 val;
        spi24bitAddr[0]=(addr & 0xff0000) >> 16;
        spi24bitAddr[1]=(addr & 0xff00) >> 8;
        spi24bitAddr[2]=(addr & 0xff);
        spi_read32bit(spi24bitAddr,&val);
        fwrite(&val,4,1,r_file_fd);
    }
    printf("read flash successed.\n");
    fclose(r_file_fd);
}

static void Write_Flash()
{
    int m_eraseNum = 0;
    FILE *w_file_fd;
    u8 Wdata[256];
    u32 addr;
    u8 spi24bitAddr[4]={0};
    int read_len;
    int ret;
    u32 i = 0;
    w_file_fd = fopen(W_FILE_NAME,"rb+");
    if(w_file_fd == NULL){
        printf("errno open file!\n");
            exit(1);
    }else 
        printf("File open successed!\n");

    ret=spi_ChipErase();
    if(ret != 0)
    {
        printf("erase failed (%s)", strerror(errno));
        exit(1);
    }
    while(!spi_ChipEraseStatus()&&(m_eraseNum < 20))
    {
        sleep(3);
        m_eraseNum++;
    }
    if(m_eraseNum == 20)
        exit(1);

    for(addr=0;addr<0x400000;addr+=256){
        spi_WriteEnable();
        spi24bitAddr[0]=(addr & 0xff0000)>> 16;
        spi24bitAddr[1]=(addr & 0xff00)>> 8;
        spi24bitAddr[2]=(addr & 0xff);
        read_len = fread(Wdata,1,256,w_file_fd);
        if(read_len == -1)
        {
            printf("file read error!\n");
            fclose(w_file_fd);
            exit(0);
        }else if(read_len == 0){
            printf("file read over!!\n");
                break;
        }else {
            printf("\033[1Aread %u * %d byte from %s\n",++i,read_len,W_FILE_NAME);
            // fflush(stdout);
            spi_Write1page(spi24bitAddr,Wdata);
        }
    }

    spi_WriteDisable();
    printf("write flash successed.\n");
    fclose(w_file_fd);
}
  
int main( int i_argc, char **pp_argv )
{ 		 
	int c;
    u8 ipflg             = 0;
    u8 portflg           = 0;
    u8 macflg            = 0;
    u8 tsportflg         = 0;
    u8 ENmulticastflg    = 0;
    u8 DISmulticastflg   = 0;
    u8 showinforflg      = 0;
    u8 inputipflg        = 0;
    u8 inputportflg      = 0;
    u8 readflg           = 0;
    u8 writeflg          = 0;
    u8 versionflg        = 0;
	u8 appversionflg     = 0;

    char *i_ip           = NULL;
    u32   i_port         = 1234;
    char *i_mac          = NULL;
    u32   i_tsport       = 1234;
    char *i_multicast    = NULL;
    char *inputip        = NULL;
    u32   inputport      = 1234;

    u16  newport         = 1234;
    u16  newTsport       = 1234;  

    u32 ltempIp;
    u8  sendBufip[4]     = {0}; 
    u8  sendBufport[2];

 
    struct timeval nTvlen = {1,0};//1S
    char recvBuf[30] = {0};
    char sendBuf[30] = "serconnectokornot";
    char tempbuf[200] = {0};
    
    u8 readMac_bytes[6];
    u8 writeMac_bytes[6];
    u8 readIp_bytes[4];
    u8 readPort_bytes[2];
    u8 readTsPort_bytes[2];
    u8 readMultip[8];
    u8 sendmulen[2];
    
    int i = 0;
    int len;
    if(i_argc == 1){
		usage();
        exit(-1);
    } 
    while ( (c = getopt_long(i_argc, pp_argv, "i:p:I:P:M:T:E:S:r:w:Dvd", long_options, NULL)) != -1 )     
	{
		switch ( c )
		{			
            case 'i':
                inputip = optarg;
                inputipflg = 1;
                break;
            case 'p':
		        inputport = strtol(optarg, NULL, 0);
                inputportflg = 1;
                break;
			case 'I':
                ipflg          = 1;
                i_ip           = optarg;
                break;
			case 'P':
		        i_port         = strtol(optarg, NULL, 0);
                portflg        = 1;
			  break;
			case 'M':
				i_mac          = optarg;
                macflg         = 1;
				break;
			case 'T':
		        i_tsport         = strtol(optarg, NULL, 0);
				tsportflg      = 1;
				break;
			case 'E':
                ENmulticastflg = 1;
                i_multicast    = optarg;
				break;
			case 'D':
                DISmulticastflg = 1;
                break; 
            case 'S':
				showinforflg   = 1;
				break;
            case 'r':
                readflg        = 1;
                R_FILE_NAME    = optarg;
                break;
            case 'w':
                writeflg       = 1;
                W_FILE_NAME    = optarg;
                break;
            case 'v':
                versionflg     = 1;
                break;
			case 'd':
                appversionflg  = 1;
                break;
				
			default:
			usage();
            exit(-1);
		}
	  }
    if(optind < i_argc ){
			usage();	
            exit(-1);
    }
	if(appversionflg  == 1 ){
		printf("This is networktool-v.1.0.0.1\n");
        exit(-1);
    }
   /*creat Socket*/
    if(inputipflg == 1 && inputportflg == 1) {
    if((sockfd = socket(AF_INET,SOCK_DGRAM,0) ) == -1){
        
        printf("creat socket failed\n");
        exit(-1);
    }
    /*time out test*/
    if(setsockopt(sockfd,SOL_SOCKET,SO_RCVTIMEO,(const char *)&nTvlen,sizeof(nTvlen)) < 0){
          printf("fail to setsockopt\n");
          exit(-1);
    }   
    if(setsockopt(sockfd,SOL_SOCKET,SO_SNDTIMEO,(const char*)&nTvlen ,sizeof(nTvlen)) < 0){
          printf("fail to  setsockopt\n");
          exit(-1);
    }
    
    /* set sockaddr_in*/
    bzero(&servaddr, sizeof(servaddr));
    servaddr.sin_family = AF_INET;
    servaddr.sin_port = htons(inputport);
    servaddr.sin_addr.s_addr = inet_addr(inputip);    
    /* sento server*/
    len = sizeof(servaddr);
    i =sendto(sockfd,sendBuf,strlen(sendBuf)+1, 0, (struct sockaddr *)&servaddr,len);
//   printf("sd:%d:%s\n",i,sendBuf);
    i = recvfrom(sockfd, recvBuf,strlen(sendBuf)+1,0,(struct sockaddr *)&servaddr,&len);
  //  printf("rv:%d:%s\n",i,recvBuf);
    for(i = 3; i < 12; i ++ ){
        if( sendBuf[i] != recvBuf[i] ){
            printf("connect failed,check your server ip and port!\n");
            return -1;
        }
    }
    printf("connect success!\n");
/*    if('q' == recvBuf[0]){
        sendto(sockfd,"q",2,0,(strcut sockaddr *)&servaddr,sizeof(servaddr))
        printf("Chat end!\n");
    }
*/
//    printf("recv from server : %s\n", recvBuf);
    // display ip and port to change edait
    if( read_24cxx(LOCAL_IP_ADDR,4,readIp_bytes) < 0){
        printf("read ip failed, not connected\n");
       // exit(-1);
    }
    printf("IP:%d.%d.%d.%d\n", readIp_bytes[0],readIp_bytes[1],readIp_bytes[2],readIp_bytes[3]);
    if(read_24cxx(LOCAL_PORT_ADDR,2,readPort_bytes) < 0){
        printf("read port failed, not connected\n");
    }
    newport = (readPort_bytes[0] << 8 | readPort_bytes[1]);
    printf("Port:%u\n", newport);
    //dispaly mac address
    if(read_24cxx(LOCAL_MAC_ADDR,6,readMac_bytes) < 0){
        printf("read mac address failed\n");
    }
    printf("MAC addr:");
    for(i = 0; i < 6; i++){
        printf("%.2x",readMac_bytes[i]);
        if(i <  5){
            printf(":");
        } 
    }
    printf("\n");
    //dispaly TsPort
    if(read_24cxx(TS_IP_PORT,2,readTsPort_bytes) < 0){
        printf("read TsPort failed!\n");
    }
    newTsport = (readTsPort_bytes[0] << 8 | readTsPort_bytes[1]);
    printf("Tsport: %u\n",newTsport);

    //mutcast setting
    if( read_24cxx(MULTICAST_EN,1,sendmulen) < 0) {//enable set{
        printf("read muticast status failed!\n");
    }
    if(sendmulen[0] == 1){
        printf("check Multcast status is  enable!\n");
        read_24cxx(MULTICAST_IP,4,readMultip);
        printf("Multicast ip: %d.%d.%d.%d\n", readMultip[0],readMultip[1],readMultip[2],readMultip[3]);
    } 
    else  printf("check Multicast status is disable!\n");
    if(macflg == 1){
    //change new mac deal
        if(sscanf(i_mac,"%2x:%2x:%2x:%2x:%2x:%2x",writeMac_bytes,writeMac_bytes+1,writeMac_bytes+2,writeMac_bytes+3,writeMac_bytes+4,writeMac_bytes+5)  <  6) {
            printf("mac tranform fail\n");
            exit(-1);
        }
        while(host_controled_by() == 1){
            printf("wait ....\n");
            sleep(1);
        }

        if( write_24cxx(LOCAL_MAC_ADDR,6,writeMac_bytes) < 0)
        {
            printf("write mac address failed\n");
        }else 
            printf("write new mac is success!\n");
        set_reset_host();
       // printf("you need to wait more than 4 minutes\n");
    }
    if(tsportflg == 1){
       printf("New Ts port %u\n", i_tsport); 
        
       sendBufport[1] = (i_tsport & 0xff);
       sendBufport[0] = (i_tsport >> 8 & 0xff);
       
       while(host_controled_by() == 1){
           printf("wait...\n");
           sleep(1);
       }
      if( write_24cxx(TS_IP_PORT,2,sendBufport) < 0){
           printf("write TsPort failed!\n");
      }else 
           printf("write new TsPort is success!\n");
       set_reset_host();
    }
    if(ENmulticastflg == 1){
        ltempIp = inet_addr(i_multicast);
        sendBufip[0] = (ltempIp & 0xff);
        sendBufip[1] = (ltempIp >> 8 & 0xff);
        sendBufip[2] = (ltempIp >> 16 & 0xff);
        sendBufip[3] = (ltempIp >> 24 & 0xff);
        printf("new Multicast ip : %d.%d.%d.%d\n", sendBufip[0],sendBufip[1],sendBufip[2],sendBufip[3]);

        while( host_controled_by() == 1 ){
            printf("wait...\n");
            sleep(1);
        }
        
        if( write_24cxx(MULTICAST_IP,4,sendBufip) < 0){
            printf("write muticast ip failed!\n");
        }
        sendBufip[0] = 1;
        if( write_24cxx(MULTICAST_EN,1,sendBufip) < 0){//enable set
            printf("enable muticast  failed!\n");
         }
        printf("Multicast set  enable!\n");
        set_reset_host();
    }else if(DISmulticastflg == 1)
    {
       // if(sendmulen[0] == 1)
       // {
        sendBufip[0] = 0;
        if(write_24cxx(MULTICAST_EN,1,sendBufip)< 0){//disable set
            printf("disenable muticast failed!\n");
        }
        printf("Multicast set  disable!\n");
		set_reset_host();
    }
    if(ipflg == 1 && portflg ==1){
         ltempIp  = inet_addr(i_ip);
         sendBufip[0] = (ltempIp & 0xff);
         sendBufip[1] = (ltempIp >> 8 & 0xff);
         sendBufip[2] = (ltempIp >> 16 & 0xff);
         sendBufip[3] = (ltempIp >> 24 & 0xff);

         printf("new ip: %d.%d.%d.%d\n", sendBufip[0],sendBufip[1],sendBufip[2],sendBufip[3]);
         while(host_controled_by() == 1){
            printf("wait ......");
            sleep(1);
        }
       if( write_24cxx(LOCAL_IP_ADDR, 4, sendBufip) < 0)
       {
           printf("write new ip failed!\n");
       }
        printf("new port:%u\n",i_port);
       // printf("%#x\n",htons(i_port));
        
        sendBufport[1] = (i_port & 0xff);
        sendBufport[0] = (i_port >> 8 & 0xff);
        
        while(host_controled_by() == 1){
            printf("wait...\n");
            sleep(1);
        }
        
        if(write_24cxx(LOCAL_PORT_ADDR,2,sendBufport) < 0){
            printf("write new port failed!\n");
        }
        set_reset_host();
        servaddr.sin_addr.s_addr = inet_addr(i_ip);
        servaddr.sin_port = htons(i_port);
    }
        /*updateFirmware*/
    if( versionflg == 1 ) {
        DisplayVersion(); 
    }           
    if( readflg == 1 ){
        Read_Flash();
    }
    if( writeflg == 1 ){
        Write_Flash();
    
    }
    } else{
        printf("warn: please input server ip and port\n");
        usage();
    }
    close(sockfd);
    return 0;  
}   
